"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const webpack_config_1 = require("@secundant/webpack-config");
const lodash_1 = require("lodash");
const ForkTsCheckerPlugin = require("fork-ts-checker-webpack-plugin");
const webpack_module_babel_1 = require("@secundant/webpack-module-babel");
const tslib_2 = require("@secundant/tslib");
const rule_1 = require("@secundant/webpack-config/utils/rule");
class TypeScript extends webpack_config_1.Module {
    getBabelLoader() {
        return {
            loader: 'babel-loader',
            options: webpack_module_babel_1.createBabelOptions(lodash_1.merge({}, webpack_module_babel_1.Babel.defaultOptions, this.babel ? this.babel.options : this.options.babel), this.session)
        };
    }
    extract() {
        const tsLoader = {
            loader: 'ts-loader',
            options: {
                transpileOnly: true
            }
        };
        const { updateBabel, babel } = this.options;
        const babelCanBeUpdated = updateBabel && babel && this.babel;
        if (babelCanBeUpdated) {
            this.babel.intercept(config => {
                const rule = rule_1.getFirstRule(config);
                const use = tslib_2.toArray(rule.use).concat(tsLoader);
                rule.test = /\.[jt]sx?$/;
                rule.use = use;
                return config;
            });
        }
        return {
            resolve: {
                extensions: ['.ts', '.tsx']
            },
            module: {
                rules: babelCanBeUpdated
                    ? []
                    : [
                        {
                            exclude: [/node_modules/],
                            test: /\.tsx?/,
                            use: lodash_1.compact([babel && this.getBabelLoader(), tsLoader])
                        }
                    ]
            },
            plugins: [
                new ForkTsCheckerPlugin({
                    checkSyntacticErrors: true
                })
            ]
        };
    }
}
TypeScript.key = 'typescript';
TypeScript.defaultOptions = {
    updateBabel: true,
    babel: {},
    ts: {}
};
tslib_1.__decorate([
    webpack_config_1.Reference()
], TypeScript.prototype, "babel", void 0);
exports.TypeScript = TypeScript;
